package com.yunos.tv.exdevicedemo;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.view.View;

import com.yunos.tv.exdeviceservice.Global;
import com.yunos.tv.exdeviceservice.amouse.AMouseEvent;
import com.yunos.tv.exdeviceservice.client.EXDevice;
import com.yunos.tv.exdeviceservice.client.EXDeviceManager;
import com.yunos.tv.exdeviceservice.client.EXDeviceManager.InitListener;
import com.yunos.tv.exdeviceservice.client.OnAMouseListener;
import com.yunos.tv.exdeviceservice.client.OnDKeyListener;
import com.yunos.tv.exdeviceservice.client.OnEXDeviceListener;
import com.yunos.tv.exdeviceservice.client.OnJMotionListener;
import com.yunos.tv.exdeviceservice.client.OnMSensorListener;
import com.yunos.tv.exdeviceservice.exdevice.EXDeviceEvent;
import com.yunos.tv.exdeviceservice.keyboard.CombKeyEvent;
import com.yunos.tv.exdeviceservice.keyboard.DKeyEvent;
import com.yunos.tv.exdeviceservice.motion.JMotionEvent;
import com.yunos.tv.exdeviceservice.sensor.MSensorEvent;

/**
 * An example of how to access the SDK with your Activity
 * 
 * @author jack.wy
 * 
 */
public class DemoActivity extends Activity implements OnAMouseListener,
		OnDKeyListener, OnJMotionListener, OnMSensorListener,
		OnEXDeviceListener, InitListener {

	private EXDeviceManager mDMgr;
	boolean bWindowFocusd = false;
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);

		/**
		 * SDK log switch, you should set Global.ENABLE_LOGD = false at least
		 * when you output the released version
		 */
		Global.ENABLE_LOGD = true;
		Global.ENABLE_LOGE = true;
		Global.ENABLE_LOGI = true;

		/**
		 * Get the singleton of the EXDeviceManager
		 */
		mDMgr = EXDeviceManager.getInstance();
	}

	Handler mHandler = new Handler();

	Runnable mRunnable = new Runnable() {

		@Override
		public void run() {
			View btn = getWindow().getDecorView();
			boolean bWindowFocusd = false;

			for (int iter : btn.getDrawableState()) {
				if (iter == android.R.attr.state_window_focused) {
					bWindowFocusd = true;
					break;
				}
			}
			Global.logD("bWindowFocusd=" + bWindowFocusd);

			mHandler.postDelayed(mRunnable, 1000);
		}

	};
	@Override
	protected void onPause() {
		if (Global.ENABLE_LOGI) {
			Global.logI("BaseActivity-onPause");
		}

		mDMgr.deinit();
		super.onPause();
	}

	@Override
	protected void onResume() {
		super.onResume();

		if (Global.ENABLE_LOGI) {
			Global.logI("BaseActivity-onResume");
		}
		mHandler.postDelayed(mRunnable, 1000);
		/**
		 * set expected property of devices to assure that you can listen the
		 * events you want, and set the direction of the device, set the
		 * features you want
		 * 
		 * noted: if you want devices work in horizontal mode, you should set
		 * the first param as true; but remember to reset to false before deinit
		 */
		mDMgr.setExpectedDeviceProperty(false, Global.DEV_FEA_AMOUSE
				| Global.DEV_FEA_KEYBOARD | Global.DEV_FEA_MOTIONSTICK
				| Global.DEV_FEA_MSENSOR | Global.DEV_FEA_VIBRATOR, false);

		mDMgr.init(getApplicationContext(), this);
		mDMgr.setOnAMouseListener(this);
		mDMgr.setOnDKeyListener(this);
		mDMgr.setOnEXDeviceListener(this);
		mDMgr.setOnJMotionListener(this);
		mDMgr.setOnMSensorListener(this);

	}

	/**
	 * the callback when SDK init successfully
	 */
	@Override
	public void onSuccess() {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onSuccess");
		}

	}

	/**
	 * the callback when SDK fail to init or after deinit
	 */
	@Override
	public void onFailed() {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onFailed");
		}

	}

	/**
	 * device event listener, when the new device enter or the existed device
	 * leave etc.
	 */
	@Override
	public void onEXDevice(EXDeviceEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onEXDevice");
		}

		EXDevice device = mDMgr.getDevice(event.getDeviceId());
		/**
		 * Noted: You may get null from getDevice by device id in this callback,
		 * because the device maybe not exist, and you just receive a
		 * disconnection event.
		 */
		if (device == null) {
			if (Global.ENABLE_LOGD) {
				Global.logD("BaseActivity-onEXDevice-the device disconnected, state: "
						+ event.getEventData().mConnectionState);
			}
		}

	}

	/**
	 * sensor event listener, you need override it if you need it
	 */
	@Override
	public void onMSensor(MSensorEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onMSensor");
		}

	}

	/**
	 * Joystick event listener, override it if you need
	 */
	@Override
	public void onJMotion(JMotionEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onJMotion");
		}

	}

	/**
	 * key event listener, override it if you need
	 */
	@Override
	public void onDKey(DKeyEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onDKey");
		}
		EXDevice dv = mDMgr.getDevice(event.getDeviceId());
		String st = dv.getKeyAction(DKeyEvent.KEYCODE_BACK) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_DPAD_UP) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_DPAD_DOWN) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_DPAD_LEFT) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_DPAD_RIGHT) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_DPAD_CENTER) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_BUTTON_A) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_BUTTON_B) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_BUTTON_X) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_BUTTON_Y) + ","
				+ dv.getKeyAction(DKeyEvent.KEYCODE_BUTTON_START) + ",";

		Global.logD("BaseActivity-onDKey" + st);
	}

	/**
	 * Combination keys event listener, you need to register combination keys
	 * first
	 */
	@Override
	public void onCombKey(CombKeyEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onCombKey");
		}

	}

	/**
	 * Mouse event listener, override it if you need
	 */
	@Override
	public void onAMouse(AMouseEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("BaseActivity-onAMouse");
		}

	}

}
