package com.yunos.tv.exdevicedemo;

import java.util.List;
import java.util.Random;

import android.content.Context;
import android.util.AttributeSet;
import android.util.TypedValue;
import android.view.LayoutInflater;
import android.view.View;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.TextView;
import android.widget.Toast;

import com.yunos.tv.exdeviceservice.Global;
import com.yunos.tv.exdeviceservice.amouse.AMouseEvent;
import com.yunos.tv.exdeviceservice.amouse.AMouseEvent.AMouseData;
import com.yunos.tv.exdeviceservice.client.EXDevice;
import com.yunos.tv.exdeviceservice.client.EXDeviceManager;
import com.yunos.tv.exdeviceservice.keyboard.CombKey;
import com.yunos.tv.exdeviceservice.keyboard.DKeyEvent;
import com.yunos.tv.exdeviceservice.keyboard.DKeyEvent.DKeyData;
import com.yunos.tv.exdeviceservice.motion.JMotionEvent;
import com.yunos.tv.exdeviceservice.motion.JMotionEvent.JMotionData;
import com.yunos.tv.exdeviceservice.sensor.MSensorEvent;
import com.yunos.tv.exdeviceservice.sensor.MSensorEvent.MSensorData;
import com.yunos.tv.exdeviceservice.touch.MTouchEvent;
import com.yunos.tv.exdeviceservice.touch.STouchEvent;

public class DeviceItemView extends LinearLayout {

	private Context mContext;

	private View mRootView;
	private TextView mMSensorData;
	private TextView mJMotionData;
	private TextView mDKeyData;
	private TextView mCombKeyData;
	private TextView mAMouseData;

	private TextView mDeviceType;
	private TextView mDeviceId;
	private TextView mVendorId;
	private TextView mProductId;
	private TextView mVendorName;
	private TextView mProductName;
	private TextView mFeatures;
	private TextView mConnectState;
	private TextView mPowerPercent;

	private int mDId = -1;
	private Random mRandom;

	public DeviceItemView(Context context) {
		super(context);
		init(context);
	}

	public DeviceItemView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context);

	}

	public void setDeviceId(int did) {
		mDId = did;
	}

	public int getDeviceId() {
		return mDId;
	}

	public void updateMSensorData(MSensorData d) {
		if (d == null) {
			return;
		}
		StringBuilder sb = new StringBuilder();
		sb.append("ax=").append(d.getValue(MSensorEvent.ACCE_AXIS_X))
				.append(", ay=");
		sb.append(d.getValue(MSensorEvent.ACCE_AXIS_Y)).append(", az=");
		sb.append(d.getValue(MSensorEvent.ACCE_AXIS_Z)).append(", gx=");
		sb.append(d.getValue(MSensorEvent.GYRO_AXIS_X)).append(", gy=");
		sb.append(d.getValue(MSensorEvent.GYRO_AXIS_Y)).append(", gz=");
		sb.append(d.getValue(MSensorEvent.GYRO_AXIS_Z)).append(", mx=");
		sb.append(d.getValue(MSensorEvent.MAGN_AXIS_X)).append(", my=");
		sb.append(d.getValue(MSensorEvent.MAGN_AXIS_Y)).append(", mz=");
		sb.append(d.getValue(MSensorEvent.MAGN_AXIS_Z));
		mMSensorData.setText(sb.toString());
	}

	public void updateJMotionData(JMotionData d) {
		if (d == null) {
			return;
		}
		StringBuilder sb = new StringBuilder();
		sb.append("lx=").append(d.getValue(JMotionEvent.AXIS_LX))
				.append(", ly=");
		sb.append(d.getValue(JMotionEvent.AXIS_LY)).append(", rx=");
		sb.append(d.getValue(JMotionEvent.AXIS_RX)).append(", ry=");
		sb.append(d.getValue(JMotionEvent.AXIS_RY)).append(", lt=");
		sb.append(d.getValue(JMotionEvent.AXIS_LT)).append(", rt=");
		sb.append(d.getValue(JMotionEvent.AXIS_RT)).append(", hx=");
		sb.append(d.getValue(JMotionEvent.AXIS_HAT_X)).append(", hy=");
		sb.append(d.getValue(JMotionEvent.AXIS_HAT_Y));
		mJMotionData.setText(sb.toString());

	}

	public void updateDKeyData(DKeyData d) {
		if (d == null) {
			return;
		}
		StringBuilder sb = new StringBuilder();
		int l = d.mSize;
		for (int i = 0; i < l; ++i) {
			sb.append(" keyCode ").append(d.mKeyCodes[i])
					.append(" action is ").append(d.mActions[i]);
		}
		if (Global.ENABLE_LOGD) {
			Global.logD("updateDKeyData: " + sb.toString());
		}
		mDKeyData.setText(sb.toString());
	}

	public void updateAMouseData(AMouseData d) {
		if (d == null) {
			return;
		}
		StringBuilder sb = new StringBuilder();
		sb.append("posX=").append(d.mPosX).append(", posY=").append(d.mPosY)
				.append(", leftKeyState=")
				.append(d.mKeyStates[AMouseEvent.MOUSE_LEFT_KEY])
				.append(", rightKeyState=")
				.append(d.mKeyStates[AMouseEvent.MOUSE_RIGHT_KEY]);
		mAMouseData.setText(sb.toString());
		if (Global.ENABLE_LOGD) {
			Global.logD("updateAMouseData: " + sb.toString());
		}
	}

	public void updateCombKeyData(CombKey ck) {
		if (ck == null) {
			return;
		}
		StringBuilder sb = new StringBuilder();
		sb.append("the CombKey id=").append(ck.getId());
		List<Integer> li = ck.getKeys();
		if (li != null) {
			int l = li.size();
			if (l > 0) {
				sb.append(", and the keys area: ");
				for (int i = 0; i < l; ++i) {
					sb.append(li.get(i)).append(" ");
				}
			}
		}
		mCombKeyData.setText(sb.toString());
	}

	public void updateEXDeviceInfo() {
		EXDevice d = EXDeviceManager.getInstance().getDevice(mDId);
		if (d != null) {
			mDeviceType.setText(d.getDevicetType() + "");
			mDeviceId.setText(d.getDeviceId() + "");
			mVendorId.setText(d.getVendorId() + "");
			mProductId.setText(d.getProductId() + "");
			mVendorName.setText(d.getVendorName());
			mProductName.setText(d.getProductName());
			mFeatures.setText(d.getDeviceFeatures() + "");
			mConnectState.setText(d.getConnState() + "");
			mPowerPercent.setText(d.getPowerPercent() + "");

			if (Global.ENABLE_LOGI) {
				Global.logI("updateEXDeviceInfo, features="
						+ d.getDeviceFeatures());
			}

		}


	}

	private void init(Context context) {
		mContext = context;
		mRootView = LayoutInflater.from(context).inflate(R.layout.device_item,
				null);
		addView(mRootView);

		mMSensorData = (TextView) mRootView.findViewById(R.id.MSensorData);
		mJMotionData = (TextView) mRootView.findViewById(R.id.JMotionData);
		mDKeyData = (TextView) mRootView.findViewById(R.id.DKeyData);
		mCombKeyData = (TextView) mRootView.findViewById(R.id.CombKeyData);
		mAMouseData = (TextView) mRootView.findViewById(R.id.AMouseData);

		mDeviceType = (TextView) mRootView.findViewById(R.id.DeviceType);
		mDeviceId = (TextView) mRootView.findViewById(R.id.DeviceId);
		mVendorId = (TextView) mRootView.findViewById(R.id.VendorId);
		mProductId = (TextView) mRootView.findViewById(R.id.ProductId);
		mVendorName = (TextView) mRootView.findViewById(R.id.VendorName);
		mProductName = (TextView) mRootView.findViewById(R.id.ProductName);
		mFeatures = (TextView) mRootView.findViewById(R.id.Features);
		mConnectState = (TextView) mRootView.findViewById(R.id.ConnectState);
		mPowerPercent = (TextView) mRootView.findViewById(R.id.PowerPercent);

		mRandom = new Random(System.currentTimeMillis());

		Button BtnVibrator = (Button) mRootView.findViewById(R.id.BtnVibrator);
		BtnVibrator.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View arg0) {
				EXDevice d = EXDeviceManager.getInstance().getDevice(mDId);
				if (d != null) {
					int type = mRandom.nextInt(4);
					int duration = mRandom.nextInt(20000);
					int mode = mRandom.nextInt(256);
					if (duration > 18000) {
						duration = -1;
					}
					d.setVibrate(type, mode, duration);
					showToast("call setVibrate, input params: vibratorType="
							+ type + ", duration=" + duration + ", mode="
							+ mode);
				}

			}
		});
		

		Button BtnFeature = (Button) mRootView.findViewById(R.id.BtnFeature);
		BtnFeature.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View arg0) {
				EXDevice d = EXDeviceManager.getInstance().getDevice(mDId);
				if (d != null) {
					int t = mRandom.nextInt(2);
					boolean b = (t == 0 ? false : true);
					d.setEnableFeatures(Global.DEV_FEA_AMOUSE
							| Global.DEV_FEA_MSENSOR
							| Global.DEV_FEA_MIC, b);
					showToast("call setEnableFeatures, switch amouse, mic, msensor: "
							+ b);
				}
				
			}
		});


		Button BtnSwitchSysMouse = (Button) mRootView
				.findViewById(R.id.BtnSwitchSysMouse);
		BtnSwitchSysMouse.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View arg0) {
				EXDevice d = EXDeviceManager.getInstance().getDevice(mDId);
				if (d != null) {
					int t = mRandom.nextInt(2);
					boolean b = (t == 0 ? false : true);
					d.setEnableSysMouse(b);

					showToast("call setEnableSysMouse : " + b);
				}
				
			}
		});
		
		Button BtnSwitchOrientation = (Button) mRootView
				.findViewById(R.id.BtnSwitchOrientation);
		BtnSwitchOrientation.setOnClickListener(new OnClickListener() {
			
			@Override
			public void onClick(View arg0) {
				EXDevice d = EXDeviceManager.getInstance().getDevice(mDId);
				if (d != null) {
					int t = mRandom.nextInt(2);
					boolean b = (t == 0 ? false : true);
					boolean ret = d.setOrientationState(b);

					showToast("call setOrientationState, set to "
							+ (b ? "Vertical" : "Horizontal") + ", return: "
							+ ret);
				}
				
			}
		});
		
		Button BtnSysEvent = (Button) mRootView.findViewById(R.id.BtnSysEvent);
		BtnSysEvent.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View arg0) {
				EXDevice d = EXDeviceManager.getInstance().getDevice(mDId);
				if (d != null) {

					int c = mRandom.nextInt(DKeyEvent.LAST_KEYCODE + 1);
					DKeyEvent ke = new DKeyEvent();
					ke.getEventData().mKeyCodes[0] = c;
					ke.getEventData().mActions[0] = DKeyEvent.ACTION_DOWN;
					ke.getEventData().mKeyCodes[1] = c;
					ke.getEventData().mActions[1] = DKeyEvent.ACTION_UP;
					ke.setDeviecId(d.getDeviceId());
					ke.getEventData().mSize = 2;
					d.sendSysKeyEvent(ke);

					showToast("call sendSysKeyEvent, : " + ke);

					AMouseEvent me = new AMouseEvent();
					int x = (int) complexToDimension(mContext,
							TypedValue.COMPLEX_UNIT_DIP, 360);
					int y = (int) complexToDimension(mContext,
							TypedValue.COMPLEX_UNIT_DIP, 100);
					me.setDeviecId(d.getDeviceId());
					me.getEventData().mPosX = x;
					me.getEventData().mPosY = y;
					me.getEventData().mKeyStates[AMouseEvent.MOUSE_LEFT_KEY] = AMouseEvent.MOUSE_PRESSED;
					me.getEventData().mKeyStates[AMouseEvent.MOUSE_RIGHT_KEY] = AMouseEvent.MOUSE_INVALID_DATA;
					d.sendSysMouseEvent(me);

					me.getEventData().mPosX = x + 10;
					me.getEventData().mPosY = y;
					d.sendSysMouseEvent(me);

					me.getEventData().mKeyStates[AMouseEvent.MOUSE_LEFT_KEY] = AMouseEvent.MOUSE_RELEASED;
					d.sendSysMouseEvent(me);

					showToast("call sendSysMouseEvent, : " + me);

					STouchEvent se = new STouchEvent();
					se.setDeviecId(d.getDeviceId());

					x = (int) complexToDimension(mContext,
							TypedValue.COMPLEX_UNIT_DIP, 360);
					y = (int) complexToDimension(mContext,
							TypedValue.COMPLEX_UNIT_DIP, 100);
					se.getEventData().mPosX = x;
					se.getEventData().mPosY = y;
					se.getEventData().mtype = STouchEvent.STOUCH_DOWN;
					d.sendSysSTouchEvent(se);

					se.getEventData().mtype = STouchEvent.STOUCH_UP;
					d.sendSysSTouchEvent(se);

					showToast("call sendSysSTouchEvent, : " + se);

					MTouchEvent mte = new MTouchEvent();
					mte.setDeviecId(d.getDeviceId());
					mte.getEventData().mPointIds[0] = 0;
					mte.getEventData().mPointIds[1] = 1;
					mte.getEventData().mPosXs[0] = (int) complexToDimension(
							mContext, TypedValue.COMPLEX_UNIT_DIP, 360);
					mte.getEventData().mPosXs[1] = (int) complexToDimension(
							mContext, TypedValue.COMPLEX_UNIT_DIP, 600);
					mte.getEventData().mPosYs[0] = (int) complexToDimension(
							mContext, TypedValue.COMPLEX_UNIT_DIP, 100);
					mte.getEventData().mPosYs[1] = (int) complexToDimension(
							mContext, TypedValue.COMPLEX_UNIT_DIP, 100);

					mte.getEventData().mAction = MTouchEvent.MTOUCH_DOWN;
					mte.getEventData().mActionIndex = 0;
					mte.getEventData().mPointCount = 1;
					d.sendSysMTouchEvent(mte);

					showToast("call sendSysMTouchEvent, : " + mte);

					mte.getEventData().mAction = MTouchEvent.MTOUCH_POINTER_DOWN;
					mte.getEventData().mActionIndex = 1;
					mte.getEventData().mPointCount = 2;
					d.sendSysMTouchEvent(mte);

					showToast("call sendSysMTouchEvent, : " + mte);

					mte.getEventData().mAction = MTouchEvent.MTOUCH_POINTER_UP;
					mte.getEventData().mActionIndex = 1;
					mte.getEventData().mPointCount = 2;
					d.sendSysMTouchEvent(mte);

					showToast("call sendSysMTouchEvent, : " + mte);

					mte.getEventData().mAction = MTouchEvent.MTOUCH_UP;
					mte.getEventData().mActionIndex = 0;
					mte.getEventData().mPointCount = 1;
					d.sendSysMTouchEvent(mte);

					showToast("call sendSysMTouchEvent, : " + mte);

				}

			}
		});

	}

	private void showToast(String str) {
		Toast.makeText(mContext, str, Toast.LENGTH_SHORT).show();
	}

	private float complexToDimension(Context context, int unit, int data) {
		return TypedValue.applyDimension(unit, data, context.getResources()
				.getDisplayMetrics());
	}

}
