package com.yunos.tv.exdevicedemo;

import java.lang.ref.WeakReference;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.Toast;

import com.yunos.tv.exdeviceservice.client.EXDeviceManager;
import com.yunos.tv.exdeviceservice.client.EXDeviceManager.InitListener;

public class MainActivity extends Activity {

	private static final int MSG_ID_TEST_INIT = 0;
	private static final int MSG_ID_TEST_DEINIT = 1;

	private static MsgHandler mMsgHandler;
	private Button mBtnEnterIn;
	private Button mBtnTestConn;
	private Button mBtnTestAlipay;

	private boolean mIsTestConnStarted = false;
	private int mTestConnCount = 0;
	private int mTestConnSuccessCount = 0;
	private int mTestConnFailedCount = 0;
	private int mTestConnCountDelay = 500;

	private static class MsgHandler extends Handler {

		WeakReference<MainActivity> mMainActivityWRef;
		InitListener mInitListener = new InitListener() {

			@Override
			public void onSuccess() {
				final MainActivity ma = mMainActivityWRef.get();
				if (ma == null) {
					return;
				}
				ma.mTestConnSuccessCount += 1;
				MainActivity.mMsgHandler.sendEmptyMessageDelayed(
						MSG_ID_TEST_DEINIT, ma.mTestConnCountDelay);

			}

			@Override
			public void onFailed() {
				final MainActivity ma = mMainActivityWRef.get();
				if (ma == null) {
					return;
				}
				ma.mTestConnFailedCount += 1;
				if (ma.mIsTestConnStarted) {
					MainActivity.mMsgHandler.sendEmptyMessageDelayed(
							MSG_ID_TEST_INIT, ma.mTestConnCountDelay);
				} else {
					ma.showToast("conn test stopped, test count="
							+ ma.mTestConnCount + ", conn count="
							+ ma.mTestConnSuccessCount + ", disconn count="
							+ ma.mTestConnFailedCount);
				}
			}
		};

		public MsgHandler(MainActivity ma) {
			mMainActivityWRef = new WeakReference<MainActivity>(ma);
		}

		public void handleMessage(Message msg) {

			if (msg == null) {
				return;
			}
			int what = msg.what;
			
			final MainActivity ma = mMainActivityWRef.get();
			if ( ma == null ){
				return;
			}

			switch (what) {
			case MSG_ID_TEST_INIT:
				if (ma.mIsTestConnStarted) {
					EXDeviceManager.getInstance().init(ma, mInitListener);

				} else {
					ma.showToast("conn test stopped, test count="
							+ ma.mTestConnCount + ", conn count="
							+ ma.mTestConnSuccessCount + ", disconn count="
							+ ma.mTestConnFailedCount);
				}
				break;
			case MSG_ID_TEST_DEINIT:
				EXDeviceManager.getInstance().deinit();
				ma.mTestConnCount += 1;

				break;
			default:
				break;
			}
		}
	}

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main_title);

		mMsgHandler = new MsgHandler(this);

		mBtnEnterIn = (Button) this.findViewById(R.id.BtnEnter);
		mBtnEnterIn.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Context c = v.getContext();
				c.startActivity(new Intent(c, TestSDKActivity.class));
				// c.startActivity(new Intent(c, DemoActivity.class));

			}
		});

		mBtnTestConn = (Button) this.findViewById(R.id.BtnTestConnection);
		mBtnTestConn.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				testConn();

			}
		});

		mBtnTestAlipay = (Button) this.findViewById(R.id.BtnAlipayTest);
		mBtnTestAlipay.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View v) {
				Context c = v.getContext();
				c.startActivity(new Intent(c,
						com.example.paydemo_new.MainActivity.class));

			}
		});

	}

	private void testConn() {
		if (!mIsTestConnStarted) {
			mBtnTestConn.setText(R.string.test_conn_stop);
			mIsTestConnStarted = true;
			mTestConnCount = 0;
			mTestConnSuccessCount = 0;
			mTestConnFailedCount = 0;
			mMsgHandler.sendEmptyMessage(MSG_ID_TEST_INIT);
		} else {
			mIsTestConnStarted = false;
			mBtnTestConn.setText(R.string.test_conn);
		}
	}

	private void showToast(String str) {
		Toast.makeText(this, str, Toast.LENGTH_SHORT).show();
	}

}
