﻿package com.yunos.tv.exdevicedemo;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.view.View.OnClickListener;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;

import com.yunos.tv.exdeviceservice.Global;
import com.yunos.tv.exdeviceservice.amouse.AMouseEvent;
import com.yunos.tv.exdeviceservice.client.BaseActivity;
import com.yunos.tv.exdeviceservice.client.EXDevice;
import com.yunos.tv.exdeviceservice.client.EXDeviceManager;
import com.yunos.tv.exdeviceservice.exdevice.EXDeviceEvent;
import com.yunos.tv.exdeviceservice.exdevice.EXDeviceEvent.EXDeviceData;
import com.yunos.tv.exdeviceservice.keyboard.CombKey;
import com.yunos.tv.exdeviceservice.keyboard.CombKeyEvent;
import com.yunos.tv.exdeviceservice.keyboard.DKeyEvent;
import com.yunos.tv.exdeviceservice.motion.JMotionEvent;
import com.yunos.tv.exdeviceservice.sensor.MSensorEvent;


/**
 * Could be an example of how to access the SDK with the BaseActivity provided
 * by the SDK
 * 
 * @author jack.wy
 * 
 */
public class TestSDKActivity extends BaseActivity {

	private boolean ENABLE_DISPLAY = true;
	private boolean ENABLE_SIMU_MOUSE = true;

	private EXDeviceManager mDMgr;

	private CombKey mDPadDownRightCK = new CombKey("DPadDownRight",
			DKeyEvent.KEYCODE_DPAD_DOWN, DKeyEvent.KEYCODE_DPAD_RIGHT);
	private CombKey mDPadUpLeftCK = new CombKey("DPadUpLeft",
			DKeyEvent.KEYCODE_DPAD_LEFT, DKeyEvent.KEYCODE_DPAD_UP);

	private CombKey mDPadUpLeftAB = new CombKey("DPadUpAB",
			DKeyEvent.KEYCODE_BUTTON_X, DKeyEvent.KEYCODE_BUTTON_Y);

	private Random mRandom;
	private List<DeviceItemView> mDIVs;
	private ViewGroup mItemsContainer;
	private TextView mDeviceCount;
	private TextView mInitState;
	private TextView mSDKVersion;
	private Button mBtnHostDeviceMode;

	private TextView mNoDisplay;
	private static String ONMSENSOR = "ONMSENSOR";
	private static String ONDEVICE = "ONDEVICE";
	private static String ONJMOTION = "ONJMOTION";
	private static String ONDKEY = "ONDKEY";
	private static String ONAMOUSE = "ONAMOUSE";
	private static String ONCOMBKEY = "ONCOMBKEY";

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.test_sdk_activity);
		/**
		 * SDK log switch, you should set Global.ENABLE_LOGD = false at least
		 * when you output the released version
		 */
		Global.ENABLE_LOGD = true;
		Global.ENABLE_LOGE = true;
		Global.ENABLE_LOGI = true;

		/**
		 * Get the singleton of the EXDeviceManager
		 */
		mDMgr = EXDeviceManager.getInstance();

		/**
		 * Register two Combination keys for test: the right arrow key + the
		 * down arrow key, and the up arrow key + the left arrow key
		 */
		mDMgr.registerCombKey(mDPadDownRightCK);
		mDMgr.registerCombKey(mDPadUpLeftCK);

		mDMgr.registerCombKey(mDPadUpLeftAB);
		mDIVs = new ArrayList<DeviceItemView>();
		mRandom = new Random(System.currentTimeMillis());

		initUI();

	}

	@Override
	protected void onDestroy() {
		/**
		 * unregister the combination keys)
		 */
		mDMgr.unregisterCombKey(mDPadDownRightCK.getId());
		mDMgr.unregisterCombKey(mDPadUpLeftCK.getId());
		super.onDestroy();
	}

	@Override
	protected void onResume() {
		super.onResume();
		/**
		 * set expected property of devices to assure that you can listen the
		 * events you want, and set the direction of the device, set the
		 * features you want
		 * 
		 * noted: if you want devices work in horizontal mode, you should set
		 * the first param as true; but remember to reset to false before deinit
		 */
		mDMgr.setExpectedDeviceProperty(false, Global.DEV_FEA_AMOUSE
				| Global.DEV_FEA_KEYBOARD | Global.DEV_FEA_MOTIONSTICK
				| Global.DEV_FEA_MSENSOR | Global.DEV_FEA_VIBRATOR, false);
		mInitState.setText(R.string.init_ing);
	}

	public void onPause() {
		Log.d("TestSDKActivity", " onpause");

		List<EXDevice> list = mDMgr.getDeviceList();
		if (list != null) {
			for (int i = 0; i < list.size(); ++i) {
				EXDevice dev = list.get(i);
				dev.setVibrate(0, 0, 0);
			}

		}
		super.onPause();
	}
	/**
	 * sensor event listener, you need override it if you need it
	 */
	@Override
	public void onMSensor(MSensorEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-MSensorEvent: " + event);
		}
		if (!ENABLE_DISPLAY) {
			mNoDisplay.setText(ONMSENSOR);
			return;
		}
		DeviceItemView item = getDItemView(event.getDeviceId());
		if (item != null) {
			item.updateMSensorData(event.getEventData());
		}
	}

	/**
	 * device event listener, when the new device enter or the existed device
	 * leave etc.
	 */
	@Override
	public void onEXDevice(EXDeviceEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-onEXDevice: " + event);
		}
		if (!ENABLE_DISPLAY) {
			mNoDisplay.setText(ONDEVICE);
			return;
		}
		DeviceItemView item = getDItemView(event.getDeviceId());
		EXDeviceData d = event.getEventData();
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-onEXDevice, connState=" + d.mConnectionState
					+ ", item=" + item + ", item counts=" + mDIVs.size());
		}
		if (item != null) {
			if (d.mConnectionState != EXDeviceEvent.DEV_STATE_DISCONNECTED) {
				item.updateEXDeviceInfo();
			} else {
				removeDItemView(item);
			}

		} else {
			if (d.mConnectionState != EXDeviceEvent.DEV_STATE_DISCONNECTED) {
				addDItemView(event.getDeviceId());
			}
		}

		updateDeviceCount();
	}

	/**
	 * Joystick event listener, override it if you need
	 */
	@Override
	public void onJMotion(JMotionEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-JMotionEvent: " + event);
		}
		if (!ENABLE_DISPLAY) {
			mNoDisplay.setText(ONJMOTION);
			return;
		}
		DeviceItemView item = getDItemView(event.getDeviceId());
		if (item != null) {
			item.updateJMotionData(event.getEventData());
		}
		/*
		 * if (ENABLE_SIMU_MOUSE) {
		 * simulateMouseEvent(mDMgr.getDevice(event.getDeviceId()), event); }
		 */
	}

	/**
	 * key event listener, override it if you need
	 */
	@Override
	public void onDKey(DKeyEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-DKeyEvent: " + event);
		}
		if (!ENABLE_DISPLAY) {
			mNoDisplay.setText(ONDKEY);
			return;
		}
		if (mDMgr.getDeviceList() != null) {
			Log.d("TestSDKActivity", "##getlist "
					+ mDMgr.getDeviceList().size());
		}
		DeviceItemView item = getDItemView(event.getDeviceId());
		if (item != null) {
			item.updateDKeyData(event.getEventData());
		}
		int did = event.getDeviceId();
		EXDevice d = mDMgr.getDevice(did);
		if (d != null && d.getDevicetType() == Global.DEV_PAIR_HAND_HANDLE) {
			int s = event.getEventData().mSize;
			if (s == 1) {
				int key = event.getEventData().mKeyCodes[0];
				if (key == DKeyEvent.KEYCODE_DPAD_LEFT
						|| key == DKeyEvent.KEYCODE_DPAD_RIGHT
						|| key == DKeyEvent.KEYCODE_DPAD_UP
						|| key == DKeyEvent.KEYCODE_DPAD_DOWN
						|| key == DKeyEvent.KEYCODE_BUTTON_A
						|| key == DKeyEvent.KEYCODE_BUTTON_B) {
					d.sendSysKeyEvent(event);
				}

				if (ENABLE_SIMU_MOUSE) {
					simulateMouseEvent(mDMgr.getDevice(event.getDeviceId()),
							event);

				}
			}

		}

	}

	/**
	 * Combination keys event listener, you need to register combination keys
	 * first
	 */
	@Override
	public void onCombKey(CombKeyEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-CombKeyEvent: " + event);
		}
		if (!ENABLE_DISPLAY) {
			mNoDisplay.setText(ONCOMBKEY);
			return;
		}
		DeviceItemView item = getDItemView(event.getDeviceId());
		if (item != null) {
			item.updateCombKeyData(event.getCombKey());
		}

		StringBuilder sb = new StringBuilder();
		sb.append("You have invoked a combination key, the id is: ").append(
				event.getCombKey().getId());
		showToast(sb.toString());
	}

	/**
	 * Mouse event listener, override it if you need
	 */
	@Override
	public void onAMouse(AMouseEvent event) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-AMouseEvent: " + event);
		}
		if (!ENABLE_DISPLAY) {
			mNoDisplay.setText(ONAMOUSE);
			return;
		}
		DeviceItemView item = getDItemView(event.getDeviceId());
		if (item != null) {
			item.updateAMouseData(event.getEventData());
		}

	}

	/**
	 * the callback when SDK init successfully
	 */
	@Override
	public void onSuccess() {
		super.onSuccess();
		updateDeviceCount();
		mInitState.setText(R.string.init_success);

	}

	/**
	 * the callback when SDK fail to init or after deinit
	 */
	@Override
	public void onFailed() {
		super.onFailed();
		updateDeviceCount();
		removeAllDItemViews();
		mInitState.setText(R.string.init_failed);
	}

	private DeviceItemView getDItemView(int deviceId) {
		if (Global.ENABLE_LOGD) {
			Global.logD("getDItemView, deviceId=" + deviceId);
		}
		if (mDIVs != null && deviceId >= 0) {
			for (DeviceItemView item : mDIVs) {
				if (Global.ENABLE_LOGD) {
					Global.logD("getDItemView, item.getDeviceId()="
							+ item.getDeviceId());
				}
				if (deviceId == item.getDeviceId()) {
					return item;
				}
			}
		}
		return null;
	}

	private void updateDeviceItemViews() {
		mItemsContainer.removeAllViews();
		int i = 0;
		int[] c = { 0xFFE0E0E0, 0xFFFFFFFF };
		for (DeviceItemView item : mDIVs) {
			if (item != null) {

				item.setBackgroundColor(c[i % 2]);
				i++;
				mItemsContainer.addView(item);
			}
		}
	}

	private void removeAllDItemViews() {
		mDIVs.clear();
		updateDeviceItemViews();
	}

	private void removeDItemView(DeviceItemView item) {
		if (Global.ENABLE_LOGD) {
			Global.logD("removeDItemView, did:" + item.getDeviceId());
		}

		mDIVs.remove(item);
		updateDeviceItemViews();
	}

	private void addDItemView(int deviceId) {
		if (Global.ENABLE_LOGD) {
			Global.logD("TestSDK-addDItemView-device id: " + deviceId);
		}
		if (deviceId < 0) {
			return;
		}
		DeviceItemView item = new DeviceItemView(this);
		item.setDeviceId(deviceId);
		int i = 0;
		int s = mDIVs.size();
		for (i = 0; i < s; ++i) {
			DeviceItemView div = mDIVs.get(i);
			if (div.getDeviceId() > deviceId) {
				mDIVs.add(i, item);
				break;
			}
		}
		if (i == s) {
			mDIVs.add(item);
		}

		updateDeviceItemViews();
		item.updateEXDeviceInfo();

	}

	private void updateDeviceCount() {
		mDeviceCount.setText("" + mDMgr.getDeviceCount());
	}

	private void updateHostDeviceMode() {
		mBtnHostDeviceMode
				.setText(mDMgr.isHostDeviceModeEnabled() ? R.string.close_hostdevicemode
						: R.string.open_hostdevicemode);
	}

	private void initUI() {
		mNoDisplay = (TextView) findViewById(R.id.NoDisplay);
		if (ENABLE_DISPLAY) {
			mNoDisplay.setVisibility(View.GONE);
		} else {
			mNoDisplay.setVisibility(View.VISIBLE);
		}
		mItemsContainer = (ViewGroup) findViewById(R.id.ItemsContainer);
		mDeviceCount = (TextView) findViewById(R.id.DeviceCount);
		mInitState = (TextView) findViewById(R.id.InitState);
		mSDKVersion = (TextView) findViewById(R.id.SDKVersion);

		mSDKVersion.setText(mDMgr.getVersionName());

		Button BtnSysEventMode = (Button) findViewById(R.id.BtnSysEventMode);
		BtnSysEventMode.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View arg0) {
				if (mDMgr.getInitState() != EXDeviceManager.STATE_INITED) {
					showToast(TestSDKActivity.this
							.getString(R.string.sdk_not_init));
					return;
				}
				int t = mRandom.nextInt(2);
				boolean b = (t == 0 ? false : true);
				mDMgr.setEnableSysEventMode(b);
				showToast("call setEnableSysEventMode: " + b);
				int type = mRandom.nextInt(4);
				int duration = mRandom.nextInt(20000);
				int mode = mRandom.nextInt(256);
				if (duration > 18000) {
					duration = -1;
				}
				List<EXDevice> list = mDMgr.getDeviceList();
				if (list != null) {
					// for (int i = 0; i < list.size(); ++i) {
					// EXDevice dev = list.get(i);
					// dev.setVibrate(type, mode, duration);
					// }
					for (int i = list.size() - 1; i >= 0; i--) {
						EXDevice dev = list.get(i);
						dev.setVibrate(type, mode, duration);
					}
				}

			}
		});

		mBtnHostDeviceMode = (Button) findViewById(R.id.BtnHostDeviceMode);
		updateHostDeviceMode();
		mBtnHostDeviceMode.setOnClickListener(new OnClickListener() {

			@Override
			public void onClick(View arg0) {
				if (mDMgr.getInitState() != EXDeviceManager.STATE_INITED) {
					showToast(TestSDKActivity.this
							.getString(R.string.sdk_not_init));
					return;
				}

				boolean b = !mDMgr.isHostDeviceModeEnabled();
				mDMgr.setEnableHostDeviceMode(b);
				updateHostDeviceMode();
				EXDevice hd = mDMgr.getHostDevice();
				showToast("call setEnableHostDeviceMode: " + b
						+ ", the host device id: "
						+ (hd != null ? hd.getDeviceId() : (-1)));

			}
		});

	}

	private void showToast(String str) {
		Toast.makeText(this, str, Toast.LENGTH_SHORT).show();
	}

	/**
	 * Example of simulating a mouse by the right joystick
	 */
	private int mDX = 0;
	private int mDY = 0;
	private int mLastJX = 640;
	private int mLastJY = 360;
	private int mSimMouseSensitivity = 10;
	private AMouseEvent mSimMouseEvent = new AMouseEvent();

	private void simulateMouseEvent(EXDevice d, JMotionEvent event) {
		int dx = (int) (event.getEventData().mAxisValues[JMotionEvent.AXIS_RX] * mSimMouseSensitivity);
		int dy = (int) (event.getEventData().mAxisValues[JMotionEvent.AXIS_RY] * mSimMouseSensitivity);

		if (Global.ENABLE_LOGD) {
			Global.logD("simulateMouseEvent, dx=" + dx + ", dy=" + dy);
		}

		if (dx * dx + dy * dy >= mDX * mDX + mDY * mDY) {

			mLastJX += dx;
			mLastJY += dy;

			mLastJX = Math.max(0, mLastJX);
			mLastJX = Math.min(mLastJX, 1280);

			mLastJY = Math.max(0, mLastJY);
			mLastJY = Math.min(mLastJY, 720);

			AMouseEvent me = mSimMouseEvent;
			int x = mLastJX;
			int y = mLastJY;
			me.setDeviecId(d.getDeviceId());
			me.getEventData().mPosX = x;
			me.getEventData().mPosY = y;
			me.getEventData().mKeyStates[AMouseEvent.MOUSE_LEFT_KEY] = AMouseEvent.MOUSE_INVALID_DATA;
			me.getEventData().mKeyStates[AMouseEvent.MOUSE_RIGHT_KEY] = AMouseEvent.MOUSE_INVALID_DATA;
			d.sendSysMouseEvent(me);

			if (Global.ENABLE_LOGD) {
				Global.logD("simulateMouseEvent, x=" + x + ", y=" + y);
			}

		}

		mDX = dx;
		mDY = dy;

	}

	private void simulateMouseEvent(EXDevice d, DKeyEvent event) {
		int s = event.getEventData().mSize;
		if (s == 1) {
			int key = event.getEventData().mKeyCodes[0];
			int action = event.getEventData().mActions[0];
			if (Global.ENABLE_LOGD) {
				Global.logD("simulateMouseEvent, key=" + key + ", action="
						+ action);
			}
			if (action == DKeyEvent.ACTION_DOWN) {
				action = AMouseEvent.MOUSE_PRESSED;
			} else if (action == DKeyEvent.ACTION_UP) {
				action = AMouseEvent.MOUSE_RELEASED;
			} else {
				return;
			}
			if (key == DKeyEvent.KEYCODE_BUTTON_L1) {
				AMouseEvent me = mSimMouseEvent;
				me.setDeviecId(d.getDeviceId());
				me.getEventData().mPosX = mLastJX;
				me.getEventData().mPosY = mLastJY;
				me.getEventData().mKeyStates[AMouseEvent.MOUSE_LEFT_KEY] = action;
				me.getEventData().mKeyStates[AMouseEvent.MOUSE_RIGHT_KEY] = AMouseEvent.MOUSE_INVALID_DATA;
				d.sendSysMouseEvent(me);
			} else if (key == DKeyEvent.KEYCODE_BUTTON_R1) {
				AMouseEvent me = mSimMouseEvent;
				me.setDeviecId(d.getDeviceId());
				me.getEventData().mPosX = mLastJX;
				me.getEventData().mPosY = mLastJY;
				me.getEventData().mKeyStates[AMouseEvent.MOUSE_LEFT_KEY] = AMouseEvent.MOUSE_INVALID_DATA;
				me.getEventData().mKeyStates[AMouseEvent.MOUSE_RIGHT_KEY] = action;
				d.sendSysMouseEvent(me);
			}
		}
	}

}
